<?php

/**
 * @file
 * Contains \Drupal\brightcove\Form\StatusOverviewForm.
 */

namespace Drupal\brightcove\Form;

use Drupal\brightcove\BrightcoveUtil;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Queue\QueueFactory;

/**
 * Class StatusOverviewForm.
 *
 * @package Drupal\brightcove\Form
 */
class StatusOverviewForm extends FormBase {

  /**
   * The queue factory.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * Constructs a StatusOverviewForm object.
   *
   * @param \Drupal\Core\Queue\QueueFactory $queueFactory
   *   The queue factory.
   * @param \Drupal\Core\Database\Connection $connection
   *   Database connection.
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   Entity type manager.
   */
  public function __construct(QueueFactory $queueFactory, Connection $connection, EntityTypeManager $entityTypeManager) {
    $this->queueFactory = $queueFactory;
    $this->connection = $connection;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * @inheritdoc
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('queue'),
      $container->get('database'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'queue_overview_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $video_num = $this->entityTypeManager->getStorage('brightcove_video')->getQuery()->count()->execute();
    $playlist_num = $this->entityTypeManager->getStorage('brightcove_playlist')->getQuery()->count()->execute();

    $counts = [
      'client' => $this->entityTypeManager->getStorage('brightcove_api_client')->getQuery()->count()->execute(),
      'video' => $video_num,
      'video_delete' => $video_num,
      'playlist' => $playlist_num,
      'playlist_delete' => $playlist_num,
      'player' => $this->entityTypeManager->getStorage('brightcove_player')->getQuery()->count()->execute(),
      'custom_field' => $this->entityTypeManager->getStorage('brightcove_custom_field')->getQuery()->count()->execute(),
    ];
    $queues = [
      'client' => $this->t('Client'),
      'video' => $this->t('Video'),
      'video_delete' => $this->t('Check deleted videos *'),
      'playlist' => $this->t('Playlist'),
      'playlist_delete' => $this->t('Check deleted playlists *'),
      'player' => $this->t('Player'),
      'custom_field' => $this->t('Custom field'),
    ];

    // There is no form element (ie. widget) in the table, so it's safe to
    // return a render array for a table as a part of the form build array.
    $form['queues'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Name'),
        $this->t('Number of entities'),
        $this->t('Item(s) in queue'),
      ],
      '#rows' => [],
    ];
    foreach ($queues as $queue => $title) {
      $form['queues']['#rows'][$queue] = [
        $title,
        $counts[$queue],
        $this->queueFactory->get("brightcove_{$queue}_queue_worker")->numberOfItems(),
      ];
    }

    $form['notice'] = [
      '#type' => 'item',
      '#markup' => '<em>* ' . $this->t('May run slowly with lots of items.') . '</em>',
    ];

    $form['sync'] = array(
      '#name' => 'sync',
      '#type' => 'submit',
      '#value' => $this->t('Sync all'),
    );
    $form['run'] = array(
      '#name' => 'run',
      '#type' => 'submit',
      '#value' => $this->t('Run all queues'),
    );
    $form['clear'] = array(
      '#name' => 'clear',
      '#type' => 'submit',
      '#value' => $this->t('Clear all queues'),
      '#description' => $this->t('Remove all items from all queues'),
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    if ($triggering_element = $form_state->getTriggeringElement()) {
      $batch_operations = [];
      $util_class = BrightcoveUtil::class;
      switch ($triggering_element['#name']) {
        case 'sync':
          $batch_operations[] = ['_brightcove_initiate_sync', []];
          // There is intentionally no break here.
        case 'run':
          $batch_operations[] = [[$util_class, 'runQueue'], ['brightcove_client_queue_worker']];
          $batch_operations[] = [[$util_class, 'runQueue'], ['brightcove_player_queue_worker']];
          $batch_operations[] = [[$util_class, 'runQueue'], ['brightcove_player_delete_queue_worker']];
          $batch_operations[] = [[$util_class, 'runQueue'], ['brightcove_custom_field_queue_worker']];
          $batch_operations[] = [[$util_class, 'runQueue'], ['brightcove_custom_field_delete_queue_worker']];
          $batch_operations[] = [[$util_class, 'runQueue'], ['brightcove_video_page_queue_worker']];
          $batch_operations[] = [[$util_class, 'runQueue'], ['brightcove_video_queue_worker']];
          $batch_operations[] = [[$util_class, 'runQueue'], ['brightcove_playlist_page_queue_worker']];
          $batch_operations[] = [[$util_class, 'runQueue'], ['brightcove_playlist_queue_worker']];
          $batch_operations[] = [[$util_class, 'runQueue'], ['brightcove_video_delete_queue_worker']];
          $batch_operations[] = [[$util_class, 'runQueue'], ['brightcove_playlist_delete_queue_worker']];
          break;

        case 'clear':
          $batch_operations[] = [[self::class, 'clearQueue'], ['brightcove_client_queue_worker']];
          $batch_operations[] = [[self::class, 'clearQueue'], ['brightcove_player_queue_worker']];
          $batch_operations[] = [[self::class, 'clearQueue'], ['brightcove_player_delete_queue_worker']];
          $batch_operations[] = [[self::class, 'clearQueue'], ['brightcove_custom_field_queue_worker']];
          $batch_operations[] = [[self::class, 'clearQueue'], ['brightcove_custom_field_delete_queue_worker']];
          $batch_operations[] = [[self::class, 'clearQueue'], ['brightcove_video_page_queue_worker']];
          $batch_operations[] = [[self::class, 'clearQueue'], ['brightcove_video_queue_worker']];
          $batch_operations[] = [[self::class, 'clearQueue'], ['brightcove_playlist_page_queue_worker']];
          $batch_operations[] = [[self::class, 'clearQueue'], ['brightcove_playlist_queue_worker']];
          $batch_operations[] = [[self::class, 'clearQueue'], ['brightcove_video_delete_queue_worker']];
          $batch_operations[] = [[self::class, 'clearQueue'], ['brightcove_playlist_delete_queue_worker']];
          break;
      }
      if ($batch_operations) {
        // Reset expired items in the default queue implementation table. If
        // that's not used, this will simply be a no-op.
        // @see system_cron()
        $this->connection->update('queue')
          ->fields(array(
            'expire' => 0,
          ))
          ->condition('expire', 0, '<>')
          ->condition('expire', REQUEST_TIME, '<')
          ->condition('name', 'brightcove_%', 'LIKE')
          ->execute();

        batch_set([
          'operations' => $batch_operations,
        ]);
      }
    }
  }

  /**
   * Clears a queue.
   *
   * @param $queue
   *   The queue name to clear.
   */
  public static function clearQueue($queue) {
    // This is a static function called by Batch API, so it's not possible to
    // use dependency injection here.
    \Drupal::queue($queue)->deleteQueue();
  }
}
